@@ -6,8 +6,5 @@ class ModuleOpenedEvent(Event):
     __module__ = 'pyobs.events'
     local = True
 
-    def __init__(self):
-        Event.__init__(self)
-
 
 __all__ = ['ModuleOpenedEvent']
