@@ -1,3 +1,5 @@
+from typing import Any
+
 from .interface import Interface
 
 
@@ -5,11 +7,11 @@ class IFocusModel(Interface):
     """The module provides a model for the telescope focus, e.g. based on temperatures."""
     __module__ = 'pyobs.interfaces'
 
-    def get_optimal_focus(self, *args, **kwargs) -> float:
+    def get_optimal_focus(self, **kwargs: Any) -> float:
         """Returns the optimal focus."""
         raise NotImplementedError
 
-    def set_optimal_focus(self, *args, **kwargs):
+    def set_optimal_focus(self, **kwargs: Any) -> None:
         """Sets optimal focus."""
         raise NotImplementedError
 
