@@ -1,3 +1,5 @@
+from typing import Any
+
 from .interface import Interface
 
 
@@ -5,7 +7,7 @@ class IReady(Interface):
     """The module can be in a "not ready" state for science and need to be initialized in some way."""
     __module__ = 'pyobs.interfaces'
 
-    def is_ready(self, *args, **kwargs) -> bool:
+    def is_ready(self, **kwargs: Any) -> bool:
         """Returns the device is "ready", whatever that means for the specific device.
 
         Returns:
