@@ -1,22 +1,22 @@
-from typing import Tuple
+from typing import Tuple, Any, Dict
 
 from .interface import Interface
-from ..utils.enums import WeatherSensors
+from pyobs.utils.enums import WeatherSensors
 
 
 class IWeather(Interface):
     """The module acts as a weather station."""
     __module__ = 'pyobs.interfaces'
 
-    def get_weather_status(self, *args, **kwargs) -> dict:
+    def get_weather_status(self, **kwargs: Any) -> Dict[str, Any]:
         """Returns status of object in form of a dictionary. See other interfaces for details."""
         raise NotImplementedError
 
-    def is_weather_good(self, *args, **kwargs) -> bool:
+    def is_weather_good(self, **kwargs: Any) -> bool:
         """Whether the weather is good to observe."""
         raise NotImplementedError
 
-    def get_current_weather(self, *args, **kwargs) -> dict:
+    def get_current_weather(self, **kwargs: Any) -> Dict[str, Any]:
         """Returns current weather.
 
         Returns:
@@ -25,7 +25,7 @@ class IWeather(Interface):
         """
         raise NotImplementedError
 
-    def get_sensor_value(self, station: str, sensor: WeatherSensors, *args, **kwargs) -> Tuple[str, float]:
+    def get_sensor_value(self, station: str, sensor: WeatherSensors, **kwargs: Any) -> Tuple[str, float]:
         """Return value for given sensor.
 
         Args:
