@@ -1,24 +1,18 @@
 import datetime
 import logging
-import math
-import os
 import threading
 import warnings
-from typing import Tuple, Optional, Dict, Any, NamedTuple
+from typing import Tuple, Optional, Dict, Any, NamedTuple, List
 import numpy as np
 from astropy.io import fits
-import astropy.units as u
+from numpy.typing import NDArray
 
-from pyobs.comm import TimeoutException, InvocationException
+from pyobs.mixins.imagegrabber import ImageGrabberMixin
 from pyobs.utils.enums import ImageType, ExposureStatus
 from pyobs.images import Image
-
-from pyobs.utils.time import Time
-from pyobs.utils.fits import format_filename
-
 from pyobs.modules import Module
 from pyobs.events import NewImageEvent, ExposureStatusChangedEvent
-from pyobs.interfaces import ICamera, IFitsHeaderProvider, IAbortable, ICameraExposureTime, IImageType
+from pyobs.interfaces import ICamera, IExposureTime, IImageType
 from pyobs.modules import timeout
 
 log = logging.getLogger(__name__)
@@ -34,19 +28,19 @@ class ExposureInfo(NamedTuple):
     exposure_time: float
 
 
-def calc_expose_timeout(camera, *args, **kwargs):
+def calc_expose_timeout(camera: IExposureTime, *args: Any, **kwargs: Any) -> float:
     """Calculates timeout for expose()."""
     return camera.get_exposure_time() + 30
 
 
-class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
+class BaseCamera(Module, ImageGrabberMixin, ICamera, IExposureTime, IImageType):
     """Base class for all camera modules."""
     __module__ = 'pyobs.modules.camera'
 
     def __init__(self, fits_headers: Optional[Dict[str, Any]] = None, centre: Optional[Tuple[float, float]] = None,
                  rotation: float = 0., flip: bool = False,
                  filenames: str = '/cache/pyobs-{DAY-OBS|date:}-{FRAMENUM|string:04d}-{IMAGETYP|type}00.fits.gz',
-                 fits_namespaces: list = None, *args, **kwargs):
+                 fits_namespaces: Optional[List[str]] = None, **kwargs: Any):
         """Creates a new BaseCamera.
 
         Args:
@@ -57,21 +51,16 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
             filenames: Template for file naming.
             fits_namespaces: List of namespaces for FITS headers that this camera should request
         """
-        Module.__init__(self, *args, **kwargs)
+        Module.__init__(self, **kwargs)
+        ImageGrabberMixin.__init__(self, fits_namespaces=fits_namespaces, fits_headers=fits_headers, centre=centre,
+                                   rotation=rotation, filenames=filenames)
 
         # check
         if self.comm is None:
             log.warning('No comm module given, will not be able to signal new images!')
 
         # store
-        self._fits_headers = fits_headers if fits_headers is not None else {}
-        if 'OBSERVER' not in self._fits_headers:
-            self._fits_headers['OBSERVER'] = ['pyobs', 'Name of observer']
-        self._centre = centre
-        self._rotation = rotation
         self._flip = flip
-        self._filenames = filenames
-        self._fits_namespaces = fits_namespaces
         self._exposure_time: float = 0.
         self._image_type = ImageType.OBJECT
 
@@ -83,11 +72,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         self._expose_lock = threading.Lock()
         self.expose_abort = threading.Event()
 
-        # night exposure number
-        self._cache = '/pyobs/modules/%s/cache.yaml' % self.name()
-        self._frame_num = 0
-
-    def open(self):
+    def open(self) -> None:
         """Open module."""
         Module.open(self)
 
@@ -96,7 +81,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
             self.comm.register_event(NewImageEvent)
             self.comm.register_event(ExposureStatusChangedEvent)
 
-    def set_exposure_time(self, exposure_time: float, *args, **kwargs):
+    def set_exposure_time(self, exposure_time: float,  **kwargs: Any) -> None:
         """Set the exposure time in seconds.
 
         Args:
@@ -108,7 +93,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         log.info('Setting exposure time to %.5fs...', exposure_time)
         self._exposure_time = exposure_time
 
-    def get_exposure_time(self, *args, **kwargs) -> float:
+    def get_exposure_time(self, **kwargs: Any) -> float:
         """Returns the exposure time in seconds.
 
         Returns:
@@ -116,7 +101,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         """
         return self._exposure_time
 
-    def set_image_type(self, image_type: ImageType, *args, **kwargs):
+    def set_image_type(self, image_type: ImageType, **kwargs: Any) -> None:
         """Set the image type.
 
         Args:
@@ -125,7 +110,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         log.info('Setting image type to %s...', image_type)
         self._image_type = image_type
 
-    def get_image_type(self, *args, **kwargs) -> ImageType:
+    def get_image_type(self, **kwargs: Any) -> ImageType:
         """Returns the current image type.
 
         Returns:
@@ -133,7 +118,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         """
         return self._image_type
 
-    def _change_exposure_status(self, status: ExposureStatus):
+    def _change_exposure_status(self, status: ExposureStatus) -> None:
         """Change exposure status and send event,
 
         Args:
@@ -142,12 +127,12 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
 
         # send event, if it changed
         if self._camera_status != status:
-            self.comm.send_event(ExposureStatusChangedEvent(self._camera_status, status))
+            self.comm.send_event(ExposureStatusChangedEvent(last=self._camera_status, current=status))
 
         # set it
         self._camera_status = status
 
-    def get_exposure_status(self, *args, **kwargs) -> ExposureStatus:
+    def get_exposure_status(self, **kwargs: Any) -> ExposureStatus:
         """Returns the current status of the camera, which is one of 'idle', 'exposing', or 'readout'.
 
         Returns:
@@ -155,7 +140,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         """
         return self._camera_status
 
-    def get_exposure_time_left(self, *args, **kwargs) -> float:
+    def get_exposure_time_left(self, **kwargs: Any) -> float:
         """Returns the remaining exposure time on the current exposure in seconds.
 
         Returns:
@@ -171,7 +156,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         diff = self._exposure.start + duration - datetime.datetime.utcnow()
         return diff.total_seconds()
 
-    def get_exposure_progress(self, *args, **kwargs) -> float:
+    def get_exposure_progress(self, **kwargs: Any) -> float:
         """Returns the progress of the current exposure in percent.
 
         Returns:
@@ -193,141 +178,6 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
             percentage = diff.total_seconds() / self._exposure[1] * 100.
             return min(percentage, 100.)
 
-    def _add_fits_headers(self, hdr: fits.Header):
-        """Add FITS header keywords to the given FITS header.
-
-        Args:
-            hdr: FITS header to add keywords to.
-        """
-
-        # convenience function to return value of keyword
-        def v(k):
-            return hdr[k][0] if isinstance(k, list) or isinstance(k, tuple) else hdr[k]
-
-        # we definitely need a DATE-OBS and IMAGETYP!!
-        if 'DATE-OBS' not in hdr:
-            log.warning('No DATE-OBS found in FITS header, adding NO further information!')
-            return
-        if 'IMAGETYP' not in hdr:
-            log.warning('No IMAGETYP found in FITS header, adding NO further information!')
-            return
-
-        # get date obs
-        date_obs = Time(hdr['DATE-OBS'])
-
-        # UT1-UTC
-        hdr['UT1_UTC'] = (float(date_obs.delta_ut1_utc), 'UT1-UTC')
-
-        # basic stuff
-        hdr['EQUINOX'] = (2000., 'Equinox of celestial coordinate system')
-
-        # pixel size in world coordinates
-        if 'DET-PIXL' in hdr and 'TEL-FOCL' in hdr and 'DET-BIN1' in hdr and 'DET-BIN2' in hdr:
-            tmp = 360. / (2. * math.pi) * v('DET-PIXL') / v('TEL-FOCL')
-            hdr['CDELT1'] = (-tmp * v('DET-BIN1'), 'Coordinate increment on x-axis [deg/px]')
-            hdr['CDELT2'] = (+tmp * v('DET-BIN2'), 'Coordinate increment on y-axis [deg/px]')
-            hdr['CUNIT1'] = ('deg', 'Units of CRVAL1, CDELT1')
-            hdr['CUNIT2'] = ('deg', 'Units of CRVAL2, CDELT2')
-            hdr['WCSAXES'] = (2, 'Number of WCS axes')
-        else:
-            log.warning('Could not calculate CDELT1/CDELT2 (DET-PIXL/TEL-FOCL/DET-BIN1/DET-BIN2 missing).')
-
-        # do we have a location?
-        if self.location is not None:
-            loc = self.location
-            # add location of telescope
-            hdr['LONGITUD'] = (float(loc.lon.degree), 'Longitude of the telescope [deg E]')
-            hdr['LATITUDE'] = (float(loc.lat.degree), 'Latitude of the telescope [deg N]')
-            hdr['HEIGHT'] = (float(loc.height.value), 'Altitude of the telescope [m]')
-
-            # add local sidereal time
-            if self.observer is not None:
-                lst = self.observer.local_sidereal_time(date_obs)
-                hdr['LST'] = (lst.to_string(unit=u.hour, sep=':'), 'Local sidereal time')
-
-        # date of night this observation is in
-        hdr['DAY-OBS'] = (date_obs.night_obs(self.observer).strftime('%Y-%m-%d'), 'Night of observation')
-
-        # centre pixel
-        if self._centre is not None:
-            hdr['DET-CPX1'] = (self._centre[0], 'x-pixel on mechanical axis in unbinned image')
-            hdr['DET-CPX2'] = (self._centre[1], 'y-pixel on mechanical axis in unbinned image')
-        else:
-            log.warning('Could not calculate DET-CPX1/DET-CPX2 (centre not given in config).')
-
-        # reference pixel in binned image
-        if 'DET-CPX1' in hdr and 'DET-BIN1' in hdr and 'DET-CPX2' in hdr and 'DET-BIN2' in hdr:
-            # offset?
-            off_x, off_y = 0, 0
-            if 'XORGSUBF' in hdr and 'YORGSUBF' in hdr:
-                off_x = v('XORGSUBF') if 'XORGSUBF' in hdr else 0.
-                off_y = v('YORGSUBF') if 'YORGSUBF' in hdr else 0.
-            hdr['CRPIX1'] = ((v('DET-CPX1') - off_x) / v('DET-BIN1'), 'Reference x-pixel position in binned image')
-            hdr['CRPIX2'] = ((v('DET-CPX2') - off_y) / v('DET-BIN2'), 'Reference y-pixel position in binned image')
-        else:
-            log.warning('Could not calculate CRPIX1/CRPIX2 '
-                            '(XORGSUBF/YORGSUBF/DET-CPX1/TEL-CPX2/DET-BIN1/DET-BIN2) missing.')
-        # only add all this stuff for OBJECT images
-        if hdr['IMAGETYP'] not in ['dark', 'bias']:
-            # projection
-            hdr['CTYPE1'] = ('RA---TAN', 'RA in tangent plane projection')
-            hdr['CTYPE2'] = ('DEC--TAN', 'Dec in tangent plane projection')
-
-            # PC matrix: rotation only, shift comes from CDELT1/2
-            if self._rotation is not None:
-                theta_rad = math.radians(self._rotation)
-                cos_theta = math.cos(theta_rad)
-                sin_theta = math.sin(theta_rad)
-                hdr['PC1_1'] = (+cos_theta, 'Partial of first axis coordinate w.r.t. x')
-                hdr['PC1_2'] = (-sin_theta, 'Partial of first axis coordinate w.r.t. y')
-                hdr['PC2_1'] = (+sin_theta, 'Partial of second axis coordinate w.r.t. x')
-                hdr['PC2_2'] = (+cos_theta, 'Partial of second axis coordinate w.r.t. y')
-            else:
-                log.warning('Could not calculate CD matrix (rotation or CDELT1/CDELT2 missing.')
-
-        # add FRAMENUM
-        self._add_framenum(hdr)
-
-    def _add_framenum(self, hdr: fits.Header):
-        """Add FRAMENUM keyword to header
-
-        Args:
-            hdr: Header to read from and write into.
-        """
-
-        # get night from header
-        night = hdr['DAY-OBS']
-
-        # increase night exp
-        self._frame_num += 1
-
-        # do we have a cache?
-        if self._cache is not None:
-            # try to load it
-            try:
-                # load cache
-                cache = self.vfs.read_yaml(self._cache)
-
-                # get new number
-                if cache is not None and 'framenum' in cache:
-                    self._frame_num = cache['framenum'] + 1
-
-                # if nights differ, reset count
-                if cache is not None and 'night' in cache and night != cache['night']:
-                    self._frame_num = 1
-
-            except (FileNotFoundError, ValueError):
-                pass
-
-            # write file
-            try:
-                self.vfs.write_yaml({'night': night, 'framenum': self._frame_num}, self._cache)
-            except (FileNotFoundError, ValueError):
-                log.warning('Could not write camera cache file.')
-
-        # set it
-        hdr['FRAMENUM'] = self._frame_num
-
     def _expose(self, exposure_time: float, open_shutter: bool, abort_event: threading.Event) -> Image:
         """Actually do the exposure, should be implemented by derived classes.
 
@@ -359,16 +209,9 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         Raises:
             ValueError: If exposure was not successful.
         """
-        fits_header_futures = {}
-        if self.comm:
-            # get clients that provide fits headers
-            clients = self.comm.clients_with_interface(IFitsHeaderProvider)
 
-            # create and run a threads in which the fits headers are fetched
-            for client in clients:
-                log.info('Requesting FITS headers from %s...', client)
-                future = self.comm.execute(client, 'get_fits_headers', self._fits_namespaces)
-                fits_header_futures[client] = future
+        # request fits headers
+        header_futures_before = self.request_fits_headers(before=True)
 
         # open the shutter?
         open_shutter = image_type not in [ImageType.BIAS, ImageType.DARK]
@@ -377,7 +220,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         self._exposure = ExposureInfo(start=datetime.datetime.utcnow(), exposure_time=exposure_time)
         try:
             image = self._expose(exposure_time, open_shutter, abort_event=self.expose_abort)
-            if image is None:
+            if image is None or image.data is None:
                 self._exposure = None
                 return None, None
         except:
@@ -385,13 +228,17 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
             self._exposure = None
             raise
 
+        # request fits headers again
+        header_futures_after = self.request_fits_headers(before=False)
+
         # flip it?
         if self._flip:
             # do we have three dimensions in array? need this for deciding which axis to flip
             is_3d = len(image.data.shape) == 3
 
             # flip image and make contiguous again
-            image.data = np.ascontiguousarray(np.flip(image.data, axis=1 if is_3d else 0))
+            flipped: NDArray[Any] = np.flip(image.data, axis=1 if is_3d else 0)  # type: ignore
+            image.data = np.ascontiguousarray(flipped)
 
         # add HDU name
         image.header['EXTNAME'] = 'SCI'
@@ -399,48 +246,15 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         # add image type
         image.header['IMAGETYP'] = image_type.value
 
-        # get fits headers from other clients
-        for client, future in fits_header_futures.items():
-            # join thread
-            log.info('Fetching FITS headers from %s...', client)
-            try:
-                headers = future.wait()
-            except TimeoutException:
-                log.warning('Fetching FITS headers from %s timed out.', client)
-                continue
-            except InvocationException as e:
-                log.warning('Could not fetch FITS headers from %s: %s.', client, e.get_message())
-                continue
-
-            # add them to fits file
-            if headers:
-                log.info('Adding additional FITS headers from %s...' % client)
-                for key, value in headers.items():
-                    # if value is not a string, it may be a list of value and comment
-                    if type(value) is list:
-                        # convert list to tuple
-                        image.header[key] = tuple(value)
-                    else:
-                        image.header[key] = value
-
-        # add static fits headers
-        for key, value in self._fits_headers.items():
-            image.header[key] = tuple(value)
-
-        # add more fits headers
-        log.info("Adding FITS headers...")
-        self._add_fits_headers(image.header)
+        # add fits headers and format filename
+        self.add_requested_fits_headers(image, header_futures_before)
+        self.add_requested_fits_headers(image, header_futures_after)
+        self.add_fits_headers(image)
+        filename = self.format_filename(image)
 
         # don't want to save?
-        if self._filenames is None:
-            return image, None
-
-        # create a temporary filename
-        filename = format_filename(image.header, self._filenames)
-        image.header['ORIGNAME'] = (os.path.basename(filename), 'The original file name')
-        image.header['FNAME'] = (os.path.basename(filename), 'FITS file file name')
         if filename is None:
-            raise ValueError('Cannot save image.')
+            return image, None
 
         # upload file
         try:
@@ -460,7 +274,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         return image, filename
 
     @timeout(calc_expose_timeout)
-    def expose(self, broadcast: bool = True, *args, **kwargs) -> str:
+    def expose(self, broadcast: bool = True, **kwargs: Any) -> str:
         """Starts exposure and returns reference to image.
 
         Args:
@@ -470,7 +284,20 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         Returns:
             Name of image that was taken.
         """
+        warnings.warn('expose() has been replaced by grab_image() and will be removed in a future version.',
+                      DeprecationWarning)
+        return self.grab_image(broadcast, **kwargs)
 
+    @timeout(calc_expose_timeout)
+    def grab_image(self, broadcast: bool = True, **kwargs: Any) -> str:
+        """Grabs an image ans returns reference.
+
+        Args:
+            broadcast: Broadcast existence of image.
+
+        Returns:
+            Name of image that was taken.
+        """
         # acquire lock
         log.info('Acquiring exclusive lock on camera...')
         if not self._expose_lock.acquire(blocking=False):
@@ -498,7 +325,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
             log.info('Releasing exclusive lock on camera...')
             self._expose_lock.release()
 
-    def _abort_exposure(self):
+    def _abort_exposure(self) -> None:
         """Abort the running exposure. Should be implemented by derived class.
 
         Raises:
@@ -506,7 +333,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         """
         pass
 
-    def abort(self, *args, **kwargs):
+    def abort(self, **kwargs: Any) -> None:
         """Aborts the current exposure and sequence.
 
         Returns:
@@ -530,7 +357,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
             raise ValueError('Could not abort exposure.')
 
     @staticmethod
-    def set_biassec_trimsec(hdr: fits.Header, left: int, top: int, width: int, height: int):
+    def set_biassec_trimsec(hdr: fits.Header, left: int, top: int, width: int, height: int) -> None:
         """Calculates and sets the BIASSEC and TRIMSEC areas.
 
         Args:
@@ -556,7 +383,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
         # for simplicity we allow prescan/overscan only in one dimension
         if (left < is_left or left+width > is_right) and (top < is_top or top+height > is_bottom):
             log.warning('BIASSEC/TRIMSEC can only be calculated with a prescan/overscan on one axis only.')
-            return False
+            return
 
         # comments
         c1 = 'Bias overscan area [x1:x2,y1:y2] (binned)'
@@ -593,7 +420,7 @@ class BaseCamera(Module, ICamera, ICameraExposureTime, IImageType, IAbortable):
             bottom_binned = np.ceil((is_top - hdr['YORGSUBF']) / hdr['YBINNING'])
             hdr['BIASSEC'] = ('[1:%d,1:%d]' % (hdr['NAXIS1'], bottom_binned), c1)
 
-    def list_binnings(self, *args, **kwargs) -> list:
+    def list_binnings(self, **kwargs: Any) -> List[Tuple[int, int]]:
         """List available binnings.
 
         Returns:
