@@ -1,3 +1,9 @@
+"""
+Modules for focus-related tasks.
+TODO: write doc
+"""
+__title__ = 'Focus'
+
 from .focusmodel import FocusModel
 from .focusseries import AutoFocusSeries
 from .projection import AutoFocusProjection
