@@ -1,12 +1,13 @@
 import logging
-from typing import Union, List, Dict, Tuple
+from typing import Union, List, Dict, Tuple, Any
 import threading
 import numpy as np
 from scipy import optimize, ndimage
 
 from pyobs.comm import RemoteException
-from pyobs.interfaces import IFocuser, ICamera, IAutoFocus, IFilters, ICameraExposureTime, IImageType
+from pyobs.interfaces import IAutoFocus
 from pyobs.events import FocusFoundEvent
+from pyobs.interfaces.proxies import IExposureTimeProxy, IImageTypeProxy, ICameraProxy, IFocuserProxy, IFiltersProxy
 from pyobs.modules import Module
 from pyobs.modules import timeout
 from pyobs.utils.enums import ImageType
@@ -18,8 +19,8 @@ class AutoFocusProjection(Module, IAutoFocus):
     """Module for auto-focusing a telescope."""
     __module__ = 'pyobs.modules.focus'
 
-    def __init__(self, focuser: Union[str, IFocuser], camera: Union[str, ICamera], filters: Union[str, IFilters] = None,
-                 offset: bool = False, *args, **kwargs):
+    def __init__(self, focuser: Union[str, IFocuserProxy], camera: Union[str, ICameraProxy],
+                 filters: Union[str, IFiltersProxy] = None, offset: bool = False, **kwargs: Any):
         """Initialize a new auto focus system.
 
         Args:
@@ -28,7 +29,7 @@ class AutoFocusProjection(Module, IAutoFocus):
             filters: Name of IFilters, if any.
             offset: If True, offsets are used instead of absolute focus values.
         """
-        Module.__init__(self, *args, **kwargs)
+        Module.__init__(self, **kwargs)
 
         # test import
         import lmfit
@@ -53,13 +54,13 @@ class AutoFocusProjection(Module, IAutoFocus):
 
         # check focuser and camera
         try:
-            self.proxy(self._focuser, IFocuser)
-            self.proxy(self._camera, ICamera)
+            self.proxy(self._focuser, IFocuserProxy)
+            self.proxy(self._camera, ICameraProxy)
         except ValueError:
             log.warning('Either camera or focuser do not exist or are not of correct type at the moment.')
 
     @timeout(600)
-    def auto_focus(self, count: int, step: float, exposure_time: float, *args, **kwargs) -> Tuple[float, float]:
+    def auto_focus(self, count: int, step: float, exposure_time: float, **kwargs: Any) -> Tuple[float, float]:
         """Perform an auto-focus series.
 
         This method performs an auto-focus series with "count" images on each side of the initial guess and the given
@@ -81,17 +82,17 @@ class AutoFocusProjection(Module, IAutoFocus):
 
         # get focuser
         log.info('Getting proxy for focuser...')
-        focuser: IFocuser = self.proxy(self._focuser, IFocuser)
+        focuser: IFocuserProxy = self.proxy(self._focuser, IFocuserProxy)
 
         # get camera
         log.info('Getting proxy for camera...')
-        camera: ICamera = self.proxy(self._camera, ICamera)
+        camera: ICameraProxy = self.proxy(self._camera, ICameraProxy)
 
         # get filter wheel and current filter
         filter_name = 'unknown'
         if self._filters is not None:
             try:
-                filter_wheel: IFilters = self.proxy(self._filters, IFilters)
+                filter_wheel: IFiltersProxy = self.proxy(self._filters, IFiltersProxy)
                 filter_name = filter_wheel.get_filter().wait()
             except ValueError:
                 log.warning('Filter module is not of type IFilters. Could not get filter.')
@@ -134,10 +135,10 @@ class AutoFocusProjection(Module, IAutoFocus):
             if self._abort.is_set():
                 raise InterruptedError()
             try:
-                if isinstance(camera, ICameraExposureTime):
-                    camera.set_exposure_time(exposure_time)
-                if isinstance(camera, IImageType):
-                    camera.set_image_type(ImageType.FOCUS)
+                if isinstance(camera, IExposureTimeProxy):
+                    camera.set_exposure_time(exposure_time).wait()
+                if isinstance(camera, IImageTypeProxy):
+                    camera.set_image_type(ImageType.FOCUS).wait()
                 filename = camera.expose().wait()
             except RemoteException:
                 raise ValueError('Could not take image.')
@@ -196,7 +197,7 @@ class AutoFocusProjection(Module, IAutoFocus):
         # return result
         return focus[0], focus[1]
 
-    def auto_focus_status(self, *args, **kwargs) -> dict:
+    def auto_focus_status(self, **kwargs: Any) -> Dict[str, Any]:
         """Returns current status of auto focus.
 
         Returned dictionary contains a list of focus/fwhm pairs in X and Y direction.
@@ -210,7 +211,7 @@ class AutoFocusProjection(Module, IAutoFocus):
             }
 
     @timeout(20)
-    def abort(self, *args, **kwargs):
+    def abort(self, **kwargs: Any):
         """Abort current actions."""
         self._abort.set()
 
