@@ -1,36 +1,40 @@
 import logging
 import threading
 import time
-from typing import Tuple, List, Dict, Any
+from typing import Tuple, List, Dict, Any, TYPE_CHECKING, Optional
 
 from astropy.coordinates import SkyCoord
 import astropy.units as u
 
 from pyobs.events import FilterChangedEvent, InitializedEvent, TelescopeMovingEvent
-from pyobs.interfaces import IFocuser, IFitsHeaderProvider, IFilters, ITemperatures, IRaDecOffsets
+from pyobs.interfaces import IFocuser, IFitsHeaderBefore, IFilters, ITemperatures, IOffsetsRaDec
 from pyobs.mixins.fitsnamespace import FitsNamespaceMixin
 from pyobs.modules.telescope.basetelescope import BaseTelescope
 from pyobs.modules import timeout
 from pyobs.utils.enums import MotionStatus
 from pyobs.utils.threads import LockWithAbort
 from pyobs.utils.time import Time
+if TYPE_CHECKING:
+    from pyobs.utils.simulation import SimWorld
+
 
 log = logging.getLogger(__name__)
 
 
-class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHeaderProvider, ITemperatures,
+class DummyTelescope(BaseTelescope, IOffsetsRaDec, IFocuser, IFilters, IFitsHeaderBefore, ITemperatures,
                      FitsNamespaceMixin):
     """A dummy telescope for testing."""
     __module__ = 'pyobs.modules.telescope'
 
-    def __init__(self, world: 'SimWorld' = None, *args, **kwargs):
+    def __init__(self, world: Optional['SimWorld'] = None, **kwargs: Any):
         """Creates a new dummy telescope."""
-        BaseTelescope.__init__(self, *args, **kwargs, motion_status_interfaces=['ITelescope', 'IFocuser', 'IFilters'])
-        FitsNamespaceMixin.__init__(self, *args, **kwargs)
+        BaseTelescope.__init__(self, **kwargs, motion_status_interfaces=['ITelescope', 'IFocuser', 'IFilters'])
+        FitsNamespaceMixin.__init__(self, **kwargs)
 
         # init world and get telescope
+        from pyobs.utils.simulation import SimWorld
         self._world = world if world is not None else \
-            self.add_child_object({'class': 'pyobs.utils.simulation.world.SimWorld'})
+            self.add_child_object({'class': 'pyobs.utils.simulation.world.SimWorld'}, SimWorld)
         self._telescope = self._world.telescope
 
         # automatically send status updates
@@ -40,7 +44,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         self._lock_focus = threading.Lock()
         self._abort_focus = threading.Event()
 
-    def open(self):
+    def open(self) -> None:
         """Open module."""
         BaseTelescope.open(self)
 
@@ -53,7 +57,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         # init status
         self._change_motion_status(MotionStatus.IDLE)
 
-    def _move_radec(self, ra: float, dec: float, abort_event: threading.Event):
+    def _move_radec(self, ra: float, dec: float, abort_event: threading.Event) -> None:
         """Actually starts tracking on given coordinates. Must be implemented by derived classes.
 
         Args:
@@ -71,7 +75,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         # start slewing
         self.__move(ra, dec, abort_event)
 
-    def _move_altaz(self, alt: float, az: float, abort_event: threading.Event):
+    def _move_altaz(self, alt: float, az: float, abort_event: threading.Event) -> None:
         """Actually moves to given coordinates. Must be implemented by derived classes.
 
         Args:
@@ -94,7 +98,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         # start slewing
         self.__move(icrs.ra.degree, icrs.dec.degree, abort_event)
 
-    def __move(self, ra: float, dec: float, abort_event: threading.Event):
+    def __move(self, ra: float, dec: float, abort_event: threading.Event) -> None:
         """Simulate move.
 
        Args:
@@ -113,7 +117,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         while self._telescope.status == MotionStatus.SLEWING and not abort_event.is_set():
             self.closing.wait(1)
 
-    def get_focus(self, *args, **kwargs) -> float:
+    def get_focus(self, **kwargs: Any) -> float:
         """Return current focus.
 
         Returns:
@@ -122,7 +126,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         return self._telescope.focus
 
     @timeout(60)
-    def set_focus(self, focus: float, *args, **kwargs):
+    def set_focus(self, focus: float, **kwargs: Any) -> None:
         """Sets new focus.
 
         Args:
@@ -150,7 +154,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
             self._change_motion_status(MotionStatus.POSITIONED, interface='IFocuser')
             self._telescope.focus = focus
 
-    def list_filters(self, *args, **kwargs) -> List[str]:
+    def list_filters(self, **kwargs: Any) -> List[str]:
         """List available filters.
 
         Returns:
@@ -158,7 +162,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         """
         return self._telescope.filters
 
-    def get_filter(self, *args, **kwargs) -> str:
+    def get_filter(self, **kwargs: Any) -> str:
         """Get currently set filter.
 
         Returns:
@@ -166,7 +170,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         """
         return self._telescope.filter
 
-    def set_filter(self, filter_name: str, *args, **kwargs):
+    def set_filter(self, filter_name: str, **kwargs: Any) -> None:
         """Set the current filter.
 
         Args:
@@ -190,7 +194,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
             logging.info('New filter set.')
 
     @timeout(60)
-    def init(self, *args, **kwargs):
+    def init(self, **kwargs: Any) -> None:
         """Initialize telescope.
 
         Raises:
@@ -204,7 +208,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         self.comm.send_event(InitializedEvent())
 
     @timeout(60)
-    def park(self, *args, **kwargs):
+    def park(self, **kwargs: Any) -> None:
         """Park telescope.
 
         Raises:
@@ -216,7 +220,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         time.sleep(5.)
         self._change_motion_status(MotionStatus.PARKED)
 
-    def set_radec_offsets(self, dra: float, ddec: float, *args, **kwargs):
+    def set_offsets_radec(self, dra: float, ddec: float, **kwargs: Any) -> None:
         """Move an RA/Dec offset.
 
         Args:
@@ -229,7 +233,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         log.info("Moving offset dra=%.5f, ddec=%.5f", dra, ddec)
         self._telescope.set_offsets(dra, ddec)
 
-    def get_radec_offsets(self, *args, **kwargs) -> Tuple[float, float]:
+    def get_offsets_radec(self, **kwargs: Any) -> Tuple[float, float]:
         """Get RA/Dec offset.
 
         Returns:
@@ -237,7 +241,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         """
         return self._telescope.offsets
 
-    def get_radec(self, *args, **kwargs) -> Tuple[float, float]:
+    def get_radec(self, **kwargs: Any) -> Tuple[float, float]:
         """Returns current RA and Dec.
 
         Returns:
@@ -245,7 +249,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         """
         return float(self._telescope.position.ra.degree), float(self._telescope.position.dec.degree)
 
-    def get_altaz(self, *args, **kwargs) -> Tuple[float, float]:
+    def get_altaz(self, **kwargs: Any) -> Tuple[float, float]:
         """Returns current Alt and Az.
 
         Returns:
@@ -257,7 +261,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         else:
             raise ValueError('No observer given.')
 
-    def get_fits_headers(self, namespaces: List[str] = None, *args, **kwargs) -> Dict[str, Tuple[Any, str]]:
+    def get_fits_header_before(self, namespaces: Optional[List[str]] = None, **kwargs: Any) -> Dict[str, Tuple[Any, str]]:
         """Returns FITS header for the current status of this module.
 
         Args:
@@ -268,7 +272,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         """
 
         # fetch from BaseTelescope
-        hdr = BaseTelescope.get_fits_headers(self)
+        hdr = BaseTelescope.get_fits_header_before(self)
 
         # focus
         hdr['TEL-FOCU'] = (self._telescope.focus, 'Focus position [mm]')
@@ -276,7 +280,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         # finished
         return self._filter_fits_namespace(hdr, namespaces=namespaces, **kwargs)
 
-    def stop_motion(self, device: str = None, *args, **kwargs):
+    def stop_motion(self, device: Optional[str] = None, **kwargs: Any) -> None:
         """Stop the motion.
 
         Args:
@@ -284,7 +288,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         """
         pass
 
-    def get_focus_offset(self, *args, **kwargs) -> float:
+    def get_focus_offset(self, **kwargs: Any) -> float:
         """Return current focus offset.
 
         Returns:
@@ -292,7 +296,7 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         """
         return 0
 
-    def get_temperatures(self, *args, **kwargs) -> dict:
+    def get_temperatures(self, **kwargs: Any) -> Dict[str, float]:
         """Returns all temperatures measured by this module.
 
         Returns:
@@ -300,8 +304,8 @@ class DummyTelescope(BaseTelescope, IRaDecOffsets, IFocuser, IFilters, IFitsHead
         """
 
         return {
-            'M1': 10,
-            'M2': 12
+            'M1': 10.,
+            'M2': 12.
         }
 
 
