@@ -1,11 +1,9 @@
 import logging
 import threading
 import typing
-from astroplan import Observer
 
+from pyobs.interfaces.proxies import IFiltersProxy, IBinningProxy, IFlatFieldProxy, ITelescopeProxy, IRoofProxy
 from pyobs.object import get_object
-from pyobs.comm import Comm
-from pyobs.interfaces import IMotion, IFlatField, ITelescope, IRoof
 from pyobs.robotic.scripts import Script
 from pyobs.utils.skyflats.priorities.base import SkyflatPriorities
 from pyobs.utils.skyflats.scheduler import Scheduler, SchedulerItem
@@ -18,11 +16,11 @@ log = logging.getLogger(__name__)
 class SkyFlats(Script):
     """Script for scheduling and running skyflats using an IFlatField module."""
 
-    def __init__(self, roof: typing.Union[str, IRoof], telescope: typing.Union[str, ITelescope],
-                 flatfield: typing.Union[str, IFlatField], functions: dict,
+    def __init__(self, roof: typing.Union[str, IRoofProxy], telescope: typing.Union[str, ITelescopeProxy],
+                 flatfield: typing.Union[str, IFlatFieldProxy], functions: dict,
                  priorities: typing.Union[dict, SkyflatPriorities], min_exptime: float = 0.5, max_exptime: float = 5,
-                 timespan: float = 7200, filter_change: float = 30, count: int = 20, combine_binnings: bool = True,
-                 readout: dict = None, *args, **kwargs):
+                 timespan: float = 7200, filter_change: float = 30, count: int = 20, readout: dict = None,
+                 *args, **kwargs):
         """Init a new SkyFlats script.
 
         Args:
@@ -36,7 +34,6 @@ class SkyFlats(Script):
             timespan: Timespan from now that should be scheduled [s]
             filter_change: Time required for filter change [s]
             count: Number of flats to schedule
-            combine_binnings: Whether different binnings use the same functions.
             readout: Dictionary with readout times (in sec) per binning (as BxB).
         """
         Script.__init__(self, *args, **kwargs)
@@ -56,7 +53,7 @@ class SkyFlats(Script):
         self._scheduler = Scheduler(functions, priorities, self.observer,
                                     min_exptime=min_exptime, max_exptime=max_exptime,
                                     timespan=timespan, filter_change=filter_change, count=count,
-                                    combine_binnings=combine_binnings, readout=readout)
+                                    readout=readout)
 
     def can_run(self) -> bool:
         """Whether this config can currently run.
@@ -67,9 +64,9 @@ class SkyFlats(Script):
 
         # get modules
         try:
-            roof: IRoof = self.comm.proxy(self._roof, IRoof)
-            telescope: ITelescope = self.comm.proxy(self._telescope, ITelescope)
-            self.comm.proxy(self._flatfield, IFlatField)
+            roof: IRoofProxy = self.comm.proxy(self._roof, IRoofProxy)
+            telescope: ITelescopeProxy = self.comm.proxy(self._telescope, ITelescopeProxy)
+            self.comm.proxy(self._flatfield, IFlatFieldProxy)
         except ValueError:
             return False
 
@@ -91,7 +88,7 @@ class SkyFlats(Script):
         """
 
         # get proxy for flatfield
-        flatfield: IFlatField = self.comm.proxy(self._flatfield, IFlatField)
+        flatfield: IFlatFieldProxy = self.comm.proxy(self._flatfield, IFlatFieldProxy)
 
         # schedule
         log.info('Scheduling flat-fields...')
@@ -111,8 +108,12 @@ class SkyFlats(Script):
             self._check_abort(abort_event)
 
             # do flat fields
-            log.info('Performing flat-fields in %s %dx%d...', item.filter_name, item.binning, item.binning)
-            done, exp_time = flatfield.flat_field(item.filter_name, self._count, item.binning).wait()
+            log.info('Performing flat-fields in %s %dx%d...', item.filter_name, *item.binning)
+            if isinstance(flatfield, IBinningProxy):
+                flatfield.set_binning(*item.binning).wait()
+            if isinstance(flatfield, IFiltersProxy):
+                flatfield.set_filter(item.filter_name).wait()
+            done, exp_time = flatfield.flat_field(self._count).wait()
             log.info('Finished flat-fields.')
 
             # increase exposure time
