@@ -1,4 +1,5 @@
 import logging
+from typing import List, Any
 
 from .filelist import FileList
 
@@ -10,10 +11,10 @@ class TestingFileList(FileList):
     """File list for testing."""
     __module__ = 'pyobs.vfs.filelists'
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any):
         pass
 
-    def __call__(self, filename: str) -> list:
+    def __call__(self, filename: str) -> List[str]:
         return [
             __file__
         ]
